<?php

/*
 * Image tools
 */

//-----------------------------------------------------------------------------

// Get image size. Return an associative array with keys 'width' and 'height'
function image_get_size($filename, $verbose=true)
{
    $cmd = 'identify -verbose "'.$filename.'"';
    if ($verbose) Message ('Excute '.htmlspecialchars($cmd).'.');
    
    ob_start();
    passthru($cmd, &$exit_code);
    $contents = ob_get_contents();
    ob_end_clean();

    if (($exit_code != 0) || 
        !ereg('Geometry: ([0-9]+)x([0-9]+)', $contents, $regs)) 
    {
        ErreurFatale(
            'Impossible de lire les dimensions de l\'image "'.
            $filename.'".');
    }

    return Array(
        'width' => $regs[1],
        'height' => $regs[2]);
}

//-----------------------------------------------------------------------------

// Generate an image thumbnail. Return the thumbnail filename
function image_generate_thumbnail(
    $src_dir, $thumb_dir, $file, 
    $width, $height, $jpeg_quality,
    $verbose=true)
{
    $filename = $src_dir.'/'.$file;
    $thumb_filename = NomfichSansExt($file).'.jpg';
    $thumbname = $thumb_dir.'/'.$thumb_filename;
    
    $cmd  = 'convert -resize "'.$width.'x'.$height.'>" ';
    $cmd .= '-quality '.$jpeg_quality.' ';
    $cmd .= '"'.$filename.'" ';
    $cmd .= '"'.$thumbname.'"';
    if ($verbose) Message ('Excute '.htmlspecialchars($cmd).'.');
    
    ob_start();
    passthru($cmd, &$exit_code);
    $contents = ob_get_contents();
    ob_end_clean();
    
    if ($exit_code != 0) {
        $txt  = '<p>Erreur lors du redimensionnement de l\'image "'.$filename;
        $txt .= '" dans "'.$thumbname.'" (erreur '.$exit_code.') !</p>';
        $txt .= '<p>Message d\'erreur :</p>';
        $txt .= '<pre>'.$contents.'</pre>';
        ErreurFatale ($txt);
    }
    return $thumb_filename;
}

//-----------------------------------------------------------------------------
?>
