<?php

/*
 * Identification d'un type de fichier en utilisant le type MIME
 * renvoy par la commande UNIX 'file --mime'. Fonctions principales :
 * - mime_get_file_mime($file) : appelle la commande file
 * - mime_identify_file($file) : identifie un type de fichier
 * (voir les commentaires pour plus d'informations)
 */

//-----------------------------------------------------------------------------

// Try to find a MIME type with the file extensions
// Return the MIME type (full text)
function mime_get_file_extension_mime($filename)
{
    $filename = NomFichSansRep($filename);
    if (eregi('\.swf$',$filename)) return 'application/x-shockwave-flash';
    if (eregi('\.rar$',$filename)) return 'application/x-rar';
    if (eregi('\.tar\.bz2$',$filename)) return 'application/x-bzip2';
    if (eregi('\.xcf$',$filename)) return 'image/x-xcf';
    return false;
}

//-----------------------------------------------------------------------------

// Excute une commande en y ajoutant " 2>&1" pour attraper les erreurs
// Renvoie un tableau associatif avec les cls :
//   'text'    : full-text version
//   'type'    : type
//   'subtype' : subtype
//   'params'  : parameters
function mime_get_file_mime($file, $filename) {
    $cmd = 'file --mime -b '.escapeshellarg($file).' 2>/dev/null';
    ob_start();
    passthru($cmd, &$exit_code);
    $content = ob_get_contents();
    ob_end_clean();

    // Vrifie le code d'erreur
    if ($exit_code != 0) {
        $txt  = "<p>Erreur lors de la dtermination du type MIME d'un fichier (".$filename.") !</p>\n";
        $txt .= "<p>Commande = ".htmlspecialchars($cmd)."</p>\n";
        $txt .= "<p>Sortie =</p>\n<pre>".htmlspecialchars($content)."</pre>"; 
        ErreurFatale(
                'Erreur (code '.$exit_code.')  lors de '.$msg_erreur,
                $txt);
    }
    $content = trim($content);

    // If file can't file type, try to get MIME from file extensions
    if ($content == 'data') {
        $try = mime_get_file_extension_mime($filename);
        if ($try != false) $content = $try;
    }

    $tab = Array('text' => $content);
    if (ereg('^([^/]+)/([a-z-]+)$',$content,$regs)) {
        $tab['type'] = $regs[1];
        $tab['subtype'] = $regs[2];
        $tab['params'] = false;
    } else if (ereg("^([^/]+)/([a-z+-]+)[,; \t](.*)$",$content,$regs)) {
        $tab['type'] = $regs[1];
        $tab['subtype'] = $regs[2];
        $tab['params'] = $regs[3];
    } else if (ereg("^data$",$content)) {
        $tab['type'] = 'data';
        $tab['subtype'] = false;
        $tab['params'] = false;
    } else {
        ErreurFatale("Impossible de traiter la chane MIME \"".$content."\" !");
    }
    return $tab;
}

//-----------------------------------------------------------------------------

function mime_identify_octet_stream($mime) {
    switch($mime['params'])
    {
    case 'x-gzip': 
        $filename = NomFichSansRep($mime['file']);
        if (eregi('\.(tar\.gz|tgz)$',$filename)) return 'tar-gz';
        return 'gzip';
    default: return false;
    }
}

//-----------------------------------------------------------------------------

function mime_identify_application($mime) {
    switch($mime['subtype'])
    {
    case 'x-shockwave-flash': return 'swf';
    case 'x-rar': return 'rar';
    case 'x-bzip2': return 'tar-bz2';
    case 'msword': 
        $ext = ExtensionNomFich($mime['file']);
        if (strcasecmp($ext,'.xls')==0) return 'ms-excel';
        if (strcasecmp($ext,'.ppt')==0) return 'ms-powerpoint';
        return 'ms-word';
    case 'x-zip': 
        $ext = ExtensionNomFich($mime['file']);
        if (strcasecmp($ext,'.sxw')==0) return 'sxw';
        if (strcasecmp($ext,'.sxc')==0) return 'sxc';
        if (strcasecmp($ext,'.sxi')==0) return 'sxi';
        return 'zip';
    case 'pdf': return 'pdf';
    case 'postscript': return 'ps';
    case 'octet-stream': return mime_identify_octet_stream($mime);
    default: return false;
    }
}

//-----------------------------------------------------------------------------

function mime_identify_audio($mime) {
    switch($mime['subtype'])
    {
    case 'x-ogg': return 'ogg';
    case 'mpeg': return 'mp3';
    default: 
        $ext = ExtensionNomFich($mime['file']);
        if (strcasecmp($ext,'.avi')==0) return 'video';
        return 'audio';
    }
}

//-----------------------------------------------------------------------------

function mime_identify_image($mime) {
    switch($mime['subtype'])
    {
    case 'png': return 'png';
    case 'jpeg': return 'jpeg';
    case 'gif': return 'gif';
    case 'x-xcf': return 'xcf';
    default: return 'image';
    }
}

//-----------------------------------------------------------------------------

function mime_identify_text($mime) {
    switch($mime['subtype'])
    {
    case 'plain': 
        $filename = NomFichSansRep($mime['file']);
        if (eregi('\.c$',$filename)) return 'c';
        if (ereg('\.(C|c\+\+)$',$filename) || eregi('\.cpp$',$filename)) 
          return 'cpp';
        if (eregi('\.xml$',$filename)) return 'xml';
        if (eregi('\.tex$',$filename)) return 'tex';
        if (eregi('\.css$',$filename)) return 'css';
        if (eregi('\.php$',$filename)) return 'php';
        return 'text';
    case 'html': return 'html';
    default: return 'text';
    }
}

//-----------------------------------------------------------------------------

// Identifie un fichier
// - $realname : fichier sur le disque dur
// - $filename : nom du fichier (si diffrent de $realname)
function mime_identify_file($realname, $filename=false) {
    if (!$filename) $filename = $realname;
    $mime = mime_get_file_mime($realname, $filename);
    $mime['file'] = $filename;
    switch ($mime['type']) {
    case 'audio':
        $type = mime_identify_audio($mime);
        break;
    case 'image':
        $type = mime_identify_image($mime);
        break;
    case 'text':
        $type = mime_identify_text($mime);
        break;
    case 'application': 
        $type = mime_identify_application($mime);
        break;
    default:
        $type = false;
    }
    return $type;
}

//-----------------------------------------------------------------------------
?>
