<?php
//----------------------------------------------------------------------------

// Create a cms_common (global var) if no one does exist
function cms_create_common() {
    global $cms_common;
    if (gettype($cms_common)=='object') return;
    $cms_common = new cms_common();
}

//----------------------------------------------------------------------------

class cms_common
{
    var $sql_table; // Associative array
    var $cms_author;   // Array of authors
    var $cms_object;   // Array of versions
    var $cms_document;  // Array of documents
    var $cms_document_type;  // Array of document types
    var $sql_field;   // 2D Associative array: first level=table, second level=field name
    var $cfg_image; // cms_image module config
    var $tmp_dir;
    var $verbose;

    function cms_common() {
        $this->verbose = false;
        
        // Directories
        $this->tmp_dir = '/tmp';
    
        // SQL tables name
        $this->sql_table = Array(
            'version' => 'cms_version',
            'document' => 'cms_document',
            'document_type' => 'cms_document_type',
            'author' => 'utilisateur',
            'publication' => 'cms_publication',
            'section' => 'fw_menu',
            'role' => 'cms_role');
        
        // SQL fields name
        $this->sql_field['author'] = Array(
            'id' => 'noUtilisateur',
            'name' => 'nom',
            'first_name' => 'prenom'
        );

        $this->sql_field['section'] = Array(
            'id' => 'nomPage',
            'title' => 'libelle',
            'state' => 'publie',
            'min_rights' => 'droitMini',
            'max_rights' => 'droitMaxi'
        );

        // Will be filled by get_version and get_document
        $this->cms_author = Array();
        $this->cms_object = Array();
        $this->cms_document = Array();
        $this->cms_document_type = Array();
    
        // Config for cms_image
        $this->cfg_image['thumbnail']['jpeg_quality'] = 70;
        $this->cfg_image['thumbnail']['width'] = 150;
        $this->cfg_image['thumbnail']['height'] = 100;
    }

    // Get an object with its SQL key 
    function get_object($version_no,$mode='load',$check_rights=true) {
        global $cms_manager;
        if (!array_key_exists($version_no, $this->cms_object)) {
            $type_no = $this->find_object_type($version_no);
            $this->load_object($type_no,$mode,$version_no);
        }
        $obj =& $this->cms_object[$version_no];
        $doc =& $obj->document;
        if ($check_rights) {
            $rights = $GLOBALS['droit_user'];
            if (($rights<$doc->min_rights) || ($doc->max_rights<$rights)) 
            {
                $txt  = 'Erreur : vous n\'avez pas le droit d\'accder au document "';
                $txt .= $doc->title.'" !';
                ErreurFatale($txt);
            }
        }
        $pub = $this->get_published_version($doc->no);
        $published = ($pub !== false) && ($obj->version_no == $pub['version_no']);
        if (!$published && !$this->can_access_archive()) {
            $txt  = 'Erreur : vous n\'avez pas le droit d\'accder  la ';
            $txt .= 'version '.$obj->version.' du document "'.$doc->title;
            $txt .= '" (car elle n\'est pas publie) !';
            ErreurFatale($txt);
        }
        return $obj; 
    }

    // Get the published version of a document
    // Return an associative array with 'version' and 'version no'
    // (return false if the document isn't published)
    function get_published_version($document_no) {
        $tp = $this->sql_table['publication'];
        $tv = $this->sql_table['version'];
        $join = Array(Array(
            'table'=>$tv,
            $tv.'.no' => $tp.'.version_no'));
        $pub = q_select(
            Array($tv.'.version',$tv.'.no',$tp.'.date'),
            $tp,
            $tp.'.document_no='.$document_no.' AND '.$tv.'.version IS NOT NULL',
            $join);
        if (($pub === false) || (q_num_rows($pub)==0)) return false;
        $pub = q_fetch_row($pub);
        return Array(
            'version' => $pub[0], 
            'version_no' => $pub[1],
            'date' => Mysql_Timestamp2time($pub[2]));
    }

    // Get an object with the SQL version number
    function get_new_object($type_no, $mode, $arg=false) {
        $version_no = $this -> load_object($type_no, $mode, $arg);
        return $this->cms_object[$version_no];
    }

    // Get state (string) of an object with it's version
    // $pub_version is the published version of the document (-1 if no
    // version is published)
    function get_object_state($obj, $pub_version)
    {
        $diff = $pub_version - $obj->version;
        if (($pub_version < 0) || ($diff < 0)) return 'Version de travail';
        if ($diff == 0) return 'Version publie';
        return 'Version d\'archive';
    }

    // Get a new document
    function get_new_document() {
        $doc =& new cms_document('new_document');
        $this->register_document(&$doc);
        return $doc;
    }

    // Get a document with its SQL number
    function get_document($no,$mode='load') {
        if (!array_key_exists($no, $this->cms_document)) {
            $this->cms_document[$no] =& new cms_document($mode,$no);
        }
        return $this->cms_document[$no];
    }

    // Get a document type with its SQL number
    function get_document_type($no) {
        if (!array_key_exists($no, $this->cms_document_type)) {
            $this->cms_document_type[$no] =& new cms_document_type($no);
        }
        return $this->cms_document_type[$no];
    }

    // Get an auhor with its SQL number
    function get_author($no) {
        if (!array_key_exists($no, $this->cms_author)) {
            $this->cms_author[$no] = new cms_author($no);
        }
        return $this->cms_author[$no];
    }

    // Return an array of sections, each value is an associative array
    // with keys 'id' and 'title'
    function find_all_sections() {
        $rights = $GLOBALS['droit_user'];
        $f = $this->sql_field['section'];
        $where  = $f['min_rights'].'<=\''.$rights.'\'';
        $where .= ' AND \''.$rights.'\'<='.$f['max_rights'];
        $r = q_select(
            Array($f['id'], $f['title']),
            $this->sql_table['section'],
            $where, // where
            false, // join
            false, // group
            $f['title'].' ASC');
          
        $sections = Array();
        while (($row = q_fetch($r)) !== false) { 
            $sections[] = Array(
                'id' => $row[0],
                'title' => $row[1]);
        }
        return $sections;
    }

    // Find object type
    function find_object_type($version_no) {
        $join = Array();
        $join[] = Array( 
                'table' => Array('cms_document' => 'doc'), 
                'doc.no' => 'ver.document');
        $type = q_read_field(
            'doc.document_type', // SELECT
            Array($this->sql_table['version'] => 'ver'), // FROM
            Array('ver.no' => $version_no), // WHERE
            $join);
        if ($type === false)  {
            $txt = 'Impossible de trouver la version '.(integer)$version_no.' !';
            ErreurFatale($txt);
        }
        return $type;
     }

    // Search versions
    // Return an array with all versions
    function search_object(
        $limit_to_published,
        $sql_where,  // string
        $sql_order=false,
        $sql_join=Array()) 
    {
        $obj=Array();

        if ($limit_to_published) {
            $sql_join[] = Array(
                'table' => $this->sql_table['publication'],
                $this->sql_table['publication'].'.version_no' => $this->sql_table['version'].'.no'
            );
            $sql_where .= ' AND '.$this->sql_table['publication'].'.version_no IS NOT NULL';
        }

        return q_select(
            Array($this->sql_table['version'].'.no', $this->sql_table['version'].'.date'), 
            $this->sql_table['version'], 
            $sql_where, 
            $sql_join, 
            false,
            $sql_order);
    }

    // Search document types 
    // Return an array with all versions
    function search_document_types() {
        $type=Array();
        $r = q_select(
            'no', 
            $this->sql_table['document_type'],
            false,
            false,
            false,
            'title ASC'); 
        while ($row=q_fetch_array($r)) {
            $no = $row['no'];
            $type[] =& $this -> get_document_type($no);
        }
        return $type;
    }

    function register_document(&$doc) {
        $this->cms_document[$doc->no] =& $doc;
    }

    function register_object(&$obj) {
        $this->cms_object[$obj->version_no] =& $obj;
    }

    function unregister_document(&$doc) {
        unset ($this->cms_document[$doc->no]);
    }

    function unregister_object(&$obj) {
        unset ($this->cms_object[$obj->version_no]);
    }

    function read_html_form($id) {
        $val = $_REQUEST[$id];
        $val = stripcslashes($val);
        return $val;
    }

    /***************** ROLE *************************************/
    
    function can_publish() 
    { return cms_user_have_role('publish', $this->section);  }
    
    function can_create_document() 
    { return cms_user_have_role('write', $this->section);  }
    
    function can_remove_document() 
    { return cms_user_have_role('publish', $this->section);  }
    
    function can_remove_version() 
    { return cms_user_have_role('publish', $this->section);  }
    
    function can_edit_document() 
    { return cms_user_have_role('write', $this->section);  }
    
    function can_edit_version() 
    { return cms_user_have_role('write', $this->section);  }
    
    function can_access_archive() 
    { 
        return cms_user_have_role('write', $this->section)
            || cms_user_have_role('publish',$this->section);
    }

    //************************* PRIVATE PART *********************

    // Load an object
    // Return version no
    function load_object($type_no, $mode, $arg=false) {
        $type = $this->get_document_type($type_no);
        $class = $type->php_module;
        $obj =& new $class($mode, $arg);
        $this->register_object(&$obj);
        return $obj->version_no;
    }
}

//----------------------------------------------------------------------------
?>
