<?php

/*
 * Image object for Castor CMS.
 */

//-----------------------------------------------------------------------------

function cms_upload_image(&$object, $file)
{
    global $cms_common;
    $img_dir = $object->document->document_type->directory;
    $thumb_dir = $img_dir.'/thumbnail';

    if (!is_dir($thumb_dir) && !@mkdir($thumb_dir, 0700)) {
        Erreur('Impossible de crer le rpertoire "<em>'.$thumb_dir.'</em>" !');
        return false;
    }
    
    // Save image
    $info = cms_upload(&$object, $file, false);
    if ($info == false) return false;
    $object -> image = $info['filename'];
   
    // Generate thumbnail
    $cfg = $cms_common->cfg_image['thumbnail'];
    $object -> thumbnail = image_generate_thumbnail(
        $img_dir,$thumb_dir, $object->image,
        $cfg['width'], $cfg['height'], $cfg['jpeg_quality'],
        $cms_common->verbose);

    // Get image MIME
    $img = $img_dir.'/'.$object->image;
    $mime = mime_get_file_mime($img, $object->image);
    $object -> mime = $mime['text'];

    // Get image size
    $size = image_get_size($img, $cms_common->verbose);
    $object -> width = $size['width'];
    $object -> height = $size['height'];
    return true;
}

//-----------------------------------------------------------------------------

function cms_evt_insert_image(&$object)
{
    $file = cms_upload_get_file('new_image');
    if (!$file) {
        if ($object->mode=='new_document') {
            Erreur('Erreur, aucune image n\'a t saisie !');
            return false;
        } else {
            return true;
        }
    }
    return cms_upload_image($object, $file);
}

//-----------------------------------------------------------------------------

function cms_evt_update_image(&$object)
{
    $file = cms_upload_get_file('new_image');
    if (!$file) return true;
    cms_evt_remove_image($object);
    return cms_upload_image($object, $file);
}

//-----------------------------------------------------------------------------

function cms_evt_remove_image(&$object)
{
    // Si l'image est encore utilise plus d'une fois
    // ne le supprime pas
    $n = q_read_field(
        'COUNT(no)',
        $object->sql_data_table,
        Array('image' => $object->image));
    if (1<$n) return;
    
    $dir = $object->document->document_type->directory;
    $f1 = $dir.'/'.$object->image;
    $f2 = $dir.'/thumbnail/'.$object->thumbnail;
    if ($GLOBALS['cms_common']->verbose) 
        Message ("Supprime les fichiers $f1 et $f2");
    unlink($f1);
    unlink($f2);
}

//-----------------------------------------------------------------------------

class cms_image extends cms_object {
    var $text;
    var $width;
    var $height;
    var $image;
    var $thumbnail;
    var $mime;
    
    function cms_image($mode,$arg=false) {
        $this->sql_data_table='cms_image';
        $this->form_xtpl = 'form_image.xtpl';
        $this->fields = Array('text','image','width','height','thumbnail','mime');
        cms_object::cms_object($mode,$arg);
    }

    function download() {
        $file = $this->document->document_type->directory.'/'.$this->image;
        cms_download($file, $this->image);
    }

    function view() {
        $file = $this->document->document_type->directory.'/'.$this->image;
        cms_view($file, $this->image, $this->mime);
    }

    function thumbnail() {
        $file = $this->document->document_type->directory.'/thumbnail/'.$this->thumbnail;
        cms_view($file, $this->thumbnail, 'image/jpeg');
    }

    //********************* PRIVATE *************************

    function read_data_from_form() {
        global $cms_common;
        foreach ($this->fields as $field) {
            $this->$field = $cms_common->read_html_form($field);
        }
    }

    function out_detail() {
        $src = $this->get_url().'?view=1';
        $txt  = '<img width="'.$this->width.'" height="'.$this->height;
        $txt .= '" alt="'.htmlspecialchars($this->text,ENT_QUOTES);
        $txt .= '" title="'.htmlspecialchars($this->text,ENT_QUOTES);
        $txt .= '" src="'.htmlspecialchars($src,ENT_QUOTES).'" />';
        return $txt;
    }

    function out_list() {
        $src = $this->get_url().'?thumbnail=1';
        $txt  = '<img';
        $txt .= ' alt="'.htmlspecialchars($this->text,ENT_QUOTES);
        $txt .= '" src="'.htmlspecialchars($src,ENT_QUOTES).'" />';
        return $txt;
    }

    function full_text() {
        return NomfichSansExt($this->image).' '.$this->text;
    }
}

//-----------------------------------------------------------------------------
?>
