<?php
class cms_object {
    var $version_no;
    var $data_no;
    var $mode;

    // Version specific
    var $document;
    var $date;
    var $full_text;
    
    // Data specific
    var $sql_data_table;
    var $form_xtpl;    
    var $fields; // Data fields list

    // Usage :
    // ('load', $no) : load an existing object
    // ('update', $no) : update an existing version
    // ('new_version', $document) : new version of an existing document
    // ('new_document', $document) : new version of a new document
    function cms_object($mode, $arg) {
        global $cms_common;
        switch ($mode) {
        case 'new_document':
            $this->document =& $arg;
            $this->version = 1;
            break;
        case 'new_version':
            $this->document =& $arg;
            break;
        default:
        case 'update':
            $this->version_no = $cms_common->read_html_form('version_no');
            break;
        case 'load':
            $this->version_no=(integer)$arg;
            break;
        }
        $this->mode=$mode;
        $this->load();
        //print_r($this);die();
    }
    
    function load(){
        if ($this->mode!='load') {
            $this->load_from_form();
        } else {
            $this->load_from_db();
        }
    }

    function out($type) {
        switch($type) {
        case 'text': return cms_str2keywords($this->full_text());
        case 'detail': return $this -> out_detail();
        case 'list': return $this -> out_list();
        case 'form': return $this -> out_form();
        }
    }

    function get_url() {
        $url  = $GLOBALS['url_root'].'cms/'.$this->document->section;
        $url .= '/'.$this->document->url;
        $url .= '/version'.$this->version.'.html';
        return $url;
    }

    function get_icon() {
        $type = $this->document->document_type;
        if ($type->php_module=='cms_file') {
            $icon = '/images/mime/'.$this->type.'.png';
        } else {
            $icon = $type->image;
        }
        return $GLOBALS['url_root'].$icon;
    }

    function get_icon_alt() {
        return $this->document->document_type->title;
    }
   
    // Return false if fails
    function save() {
        if ($this->mode=='new_document' or $this->mode=='new_version')
            return $this->save_insert();
        else
            return $this->save_update();
    }


    // Remove this version from database
    function remove() {
        global $cms_common;

        // Use event ?
        $event = $this->document->document_type->event_remove;
        if ($event) $event(&$this);

        // Remove object from cache
        $cms_common->unregister_object($this);

        // Remove entries in DB
        q_delete(
            $this->sql_data_table,
            Array('no' => $this->data_no));
        q_delete(
            $cms_common->sql_table['version'],
            Array('no' => $this->version_no));
    }

    // ******************* PRIVATE PART **********************

    function save_insert() {
        $event = $this->document->document_type->event_insert;
        if ($event) {
            $ok = $event(&$this);
            if (!$ok) return false;
            $this->recalculate_fields();
        }
        $this->save_data_insert();
        $this->save_version_insert();
        return true;
    }
   
    // Recalculate fields (eg. full_text)
    function recalculate_fields()
    {
        $this->full_text=$this->out('text');
    }

    function save_update() {
        $event = $this->document->document_type->event_update;
        if ($event) {
            $ok = $event(&$this);
            if (!$ok) return false;
            $this->recalculate_fields();
        }        
        $this->save_data_update();
        $this->save_version_update();
        return true;
    }

    function save_version_insert() {
        global $cms_common;
        
        // Save datas in database
        $fields = Array(
            'no'=>$this->version_no,
            'version' => $this->version,
            'document'=>$this->document->no,
            'data'=>$this->data_no,
            'full_text'=>$this->full_text
        );
        q_insert(
            $cms_common->sql_table['version'],
            $fields);
    }

    function save_version_update() {
        global $cms_common;
        q_update(
            $cms_common->sql_table['version'],
            Array(
                'date'=>q_time2timestamp($this->date), 
                'full_text'=>$this->full_text),
            Array('no' => $this->version_no));
    }

    // Return an associative array with values
    // with $out: 'text', 'html' or (html) 'attribut'
    function get_fields_values($out='text') {
        $value = Array();
        foreach ($this->fields as $key) {
            $val = $this->$key;
            switch ($out) {
            case 'attribut': $val = htmlspecialchars($val, ENT_QUOTES); break;
            default: break;
            }
            $value[$key] = $val;
        }
        return $value;
    }

    function out_form() {
        global $cms_common;
        $xtpl = new XTemplate('./templates/F/cms_manager/'.$this->form_xtpl);
        $xtpl->assign('modid', $cms_common->modid);
        $xtpl->assign($this->get_fields_values('attribut'));
        $xtpl->parse('form');
        return $xtpl->text('form');
    }

    function load_from_db() {
        global $cms_common;

        $table_ver = $cms_common->sql_table['version'];
        $table_data = $this->sql_data_table;
        
        $fields_ver = Array(
            'version', 'document', 'data', 'date', 
            'full_text');
        $fields = Array();
        foreach ($this->fields as $f) $fields[] = $table_data.'.'.$f;
        foreach ($fields_ver as $f) $fields[] = $table_ver.'.'.$f;
        
        $join = Array();
        $join[] = Array(
            'table' => $table_data,
            $table_data.'.no' => $table_ver.'.data');
       
        $r=q_read_row(
            $fields,
            $table_ver,
            Array($table_ver.'.no' => $this->version_no),
            $join);
        $this->version=(integer)$r[$table_ver.'.version'];
        $this->data_no = $r[$table_ver.'.data'];        
        $this->full_text=$r[$table_ver.'.full_text'];
        $this->date=Mysql_Timestamp2time($r[$table_ver.'.date']);
        foreach ($this->fields as $field)
            $this->$field=$r[$table_data.'.'.$field];

        $document_no = $r[$table_ver.'.document'];
        $this->document =& $cms_common->get_document($document_no);

        // --- Load datas --
    }

    function save_data_insert() {
        $fields = $this->get_fields_values();
        $fields['no'] = $this->data_no;
        q_insert(
            $this->sql_data_table, 
            $fields);
    }

    function save_data_update() {
        $fields = $this->get_fields_values();
        q_update(
            $this->sql_data_table, 
            $fields, 
            Array('no' => $this->data_no));
    }

    function load_from_form() {
        global $cms_common;

        // Find empty key for data
        if ($this->mode == 'update') {
            $this->data_no =$cms_common->read_html_form('data_no'); 
        }

        // Read data form HTML form
        $this->read_data_from_form();

        // Load the relative document
        if ($this->mode != 'new_document') {
            // Charge le document relatif  cet objet
            $document_no = $cms_common->read_html_form('document_no'); 
            $this->document =& $cms_common->get_document($document_no);
        }
        
        // Lit la date actuelle
        $this->date = q_now_time();

        // Mise  jour de la version texte
        $this->recalculate_fields();

        if ($this->mode=='new_document' or $this->mode=='new_version') {
            // Find a free version key        
            $this->version_no = 1+q_read_field(
                Array('MAX(no)'), 
                $cms_common->sql_table['version']);

            // Find a free data key
            $this->data_no = 1+q_read_field(
                Array('MAX(no)'), 
                $this->sql_data_table);
        }

        if ($this->mode=='new_version') {
            $this->version = $cms_common->read_html_form('version');
        }
    }
}
