<?php

/******************************************************************************
 *   Copyright (c) 2004 The 2le development team                              *
 ******************************************************************************
 *                                                                            *
 *  This file is part of CASTOR.                                              *
 *                                                                            *
 *  CASTOR is free software; you can redistribute it and/or modify            *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation; either version 2 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  CASTOR is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with CASTOR; if not, write to the Free Software                     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                            *
 *****************************************************************************/

/*
 * CMS Document.
 */

class cms_document {
    var $no;
    var $title;
    var $author;
    var $creation_date;
    var $lang;
    var $document_type;
    var $section;
    var $url;
    var $min_rights;
    var $max_rights;

    // Modes :
    // ('load', $no) : load from DB
    // ('new_document) : new empty document
    // ('update') : update an existing document
    function cms_document($mode,$arg=false){
        switch($mode) {
        case 'new_document':
            break;
        case 'update':
        case 'load':
        default:
            $this->no=$arg;
            break;
        }
        $this->mode=$mode;
        $this->load();
    }
    
    function load(){   
        global $cms_common;
        if ($this->mode == 'load')
            $this->load_from_db();
        else
            $this->load_from_form();
    }

    function save() {
        if ($this->mode != 'new_document')
            $this->save_update();
        else
            $this->save_insert();
    }

    // Count version number for this document
    function count_version() {
        global $cms_common;
        $n = q_read_field(
            Array('COUNT(no)'),
            $cms_common->sql_table['version'],
            Array('document' => $this->no));
        if ($n === false) {
            ErreurFatale('Il n\'existe aucune version du document numro '.$this->no.' !');
        }
        return $n;
    }

    // Find last version of this document
    // Return the no of the version
    function find_last_version() {
        global $cms_common;
        $version_no = q_read_field(
            Array('MAX(no)'),
            $cms_common->sql_table['version'],
            Array('document' => $this->no));
        if ($version_no === false) {
            ErreurFatale('Il n\'existe aucune version du document numro '.$this->no.' !');
        }
        return $version_no;
    }

    // Remove this document
    function remove() {
        global $cms_common;
   
        // Remove publication
        q_delete(
            $cms_common->sql_table['publication'],
            Array('document_no' => $this->no));
   
        // Remove all versions
        $r = q_select(
            'no',
            $cms_common->sql_table['version'],
            Array('document' => $this->no));
        while ($row=q_fetch_array($r)) {
            $obj =& $cms_common -> get_object($row['no'],'load',false);
            $obj->remove();
        }

        // Remove the document
        $cms_common->unregister_document(&$this);
        q_delete(
            $cms_common->sql_table['document'],
            Array('no' => $this->no));
    }

    //*********************** PRIVATE **********************

    function save_insert(){   
        global $cms_common;
        $fields = Array(
            'no'=>$this->no,
            'title'=>$this->title,
            'author' => $this->author->no,
            'creation_date'=>'NOW()',
            'lang'=>$this->lang,
            'url'=>$this->url,
            'section'=>$this->section,
            'document_type'=>$this->document_type->no,
            'min_rights'=>$this->min_rights,
            'max_rights'=>$this->max_rights
        );
        q_insert(
            $cms_common->sql_table['document'], 
            $fields);
        $cms_common->register_document(&$this);
    }

    function save_update() {
        global $cms_common;
        $fields = Array(
            'title'=>$this->title,
            'section'=>$this->section,
            'lang'=>$this->lang,
            'url'=>$this->url,
            'min_rights'=>$this->min_rights,
            'max_rights'=>$this->max_rights
            );
        q_update(
            $cms_common->sql_table['document'], 
            $fields, 
            Array('no'=>$this->no));
    }

    function load_from_form() {
        global $cms_common;
        if ($this->mode=='new_document') {
            $this->no = 1+q_read_field(
                Array('MAX(no)'),
                $cms_common->sql_table['document']);
            $type_no = $cms_common->read_html_form('document_type');
            $author_no = $cms_common->read_html_form('document_author');
        } else {
            // update
            $this->no=$cms_common->read_html_form('document_no');
            $row = q_read_row(
                Array('document_type', 'author'),
                $cms_common->sql_table['document'],
                Array('no'=>$this->no));
            $type_no = $row['document_type'];
            $author_no = $row['author'];
        }
        
        $this->document_type =& $cms_common->get_document_type($type_no);
        $this->author =& $cms_common->get_author($author_no);
        $this->section = $cms_common->read_html_form('section');
        $this->lang = $cms_common->read_html_form('document_lang');
        $this->title = $cms_common->read_html_form('document_title');
        $this->min_rights = $cms_common->read_html_form('min_rights');
        $this->max_rights = $cms_common->read_html_form('max_rights');
        $this->url = cms_str2url($this->title);
    }

    function load_from_db() {
        global $cms_common;
        $r=q_read_row(
            Array('document_type', 'author', 
            'title', 'lang', 'section', 'url','creation_date',
            'min_rights', 'max_rights'),
            $cms_common->sql_table['document'],
            Array('no' => $this->no));
        $this->title=$r['title'];
        $this->lang=$r['lang'];
        $this->creation_date=Mysql_Timestamp2time($r['creation_date']);
        $this->section=$r['section'];
        $this->url=$r['url'];
        $this->min_rights=$r['min_rights'];
        $this->max_rights=$r['max_rights'];
        $this->document_type =& $cms_common->get_document_type($r['document_type']);
        $this->author =& $cms_common->get_author($r['author']);
    }
}
?>
