<?php

/******************************************************************************
 *   Copyright (c) 2004 The 2le development team                              *
 ******************************************************************************
 *                                                                            *
 *  This file is part of CASTOR.                                              *
 *                                                                            *
 *  CASTOR is free software; you can redistribute it and/or modify            *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation; either version 2 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  CASTOR is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with CASTOR; if not, write to the Free Software                     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                            *
 *****************************************************************************/

/*
 * MySQL tools.
 */

//---------------------------------------------------------------------------

// Convertit une chane au format MySQL. 
// with $quote: string or NULL
function q_prepare_string ($txt, $quote="'", $test_table=false)
{
    // SQL function or not quoted
    if (eregi('^(NOW|COUNT|MAX|SUM)', $txt)) return $txt;

    // Replace ' by \'
    $txt = str_replace('\'', '\\\'', $txt);

    // Add quotes if needed
    if ($quote !== NULL) {
        if ($test_table && ereg('^([^\.]+)\.(.+)$', $txt, $regs)) {
            $txt = $regs[1] . '.' . $quote . $regs[2] . $quote;
        } else {
            $txt = $quote . $txt . $quote;
        }
    }

    return $txt;  
}

//---------------------------------------------------------------------------

// Convertit une liste en une chane MySQL.
// Voyez MySQL::txt2mysql pour les valeurs possibles de $quote
function q_array2string ($array, $quote="'", $glue=', ',$test_table=false)
{
  $txt = '';
  $need_glue = false;
  foreach ($array as $key => $value)
  {
    if ($need_glue) $txt .= $glue; else $need_glue = true;
    if (gettype($key) == 'string')
        $txt .= $key.' AS '.q_prepare_string($value, $quote, $test_table);
    else
        $txt .= q_prepare_string($value, $quote, $test_table);
  }
  return $txt;
}

//---------------------------------------------------------------------------

// Prepare table string for SQL query
// if $table is an array : return `table` AS `name`, else return $table
function q_prepare_table ($table) {
    if (gettype($table)=='array')
        return q_prepare_string(key($table), "`",true).' AS '.q_prepare_string(current($table), "`");
    else
        return q_prepare_string($table,'`',true);
}

//---------------------------------------------------------------------------

// Prepare fields string for SQL query
function q_prepare_fields($fields) {
    if (gettype($fields)=='array')
        return q_array2string($fields, "`", ", ", true);
    else
        return $fields;
}

//---------------------------------------------------------------------------

// Prepare WHERE statement for SQL query
function q_prepare_where($where) {
    if (gettype($where)=='array') {
        return 
            "\nWHERE "
            .q_associative_array2string($where,'=',' AND ',"`","'",true);
    }
    if ($where !== false) return "\nWHERE ".$where;
    return '';
}

//---------------------------------------------------------------------------

// Prepare ORDER BY statement for SQL query
function q_prepare_order($order) {
    if ($order !== false) return "\nORDER BY ".$order;
    return '';
}

//---------------------------------------------------------------------------

// Retourne `key1`$join'value1'$glue(...) pour tous les lments 
// du tableau associatif $valeurs
function q_associative_array2string (
    $fields,$join='=',$glue=', ',
    $quote1="`",$quote2=NULL,$test_table1=false,$test_table2=false)
{
  $need_glue = false;
  $out = '';
  foreach ($fields as $key => $value)
  {
    if ($need_glue) $out .= $glue; else $need_glue = true;
    $out .= q_prepare_string($key,$quote1,$test_table1) . $join . q_prepare_string($value,$quote2,$test_table2);
  }
  return $out;
}
  
//---------------------------------------------------------------------------

// Parse a field to find the key name
function q_parse_key($field) {
    // field as name => key=name
    if (gettype($field)=='array') return current($field);
        
    // table.field => key=field
    if (ereg('^[^.]+\.(.+)$', $field, $regs)) return $regs[1];
        
    // "field as name" => key=name
    if (eregi('^[^ ]+ as ([^ ]+)$', $field, $regs)) return $regs[1];
        
    return $field;
}


//-----------------------------------------------------------------------------

// Convertit une date MySQL dans le format PHP
function Mysql_Timestamp2time($date)
{
  $txt  = "^([0-9]{4})"; // Anne
  $txt .="([0-9]{2})"; // Mois
  $txt .= "([0-9]{2})"; // Jour
  $txt .= "([0-9]{2})"; // Heure 
  $txt .= "([0-9]{2})"; // Minute 
  $txt .= "([0-9]{2})$"; // Seconde
  if (ereg($txt, $date, $regs)) {
    $annee = $regs[1];
    $mois = $regs[2];
    $jour = $regs[3];
    $heure = $regs[4];
    $minute = $regs[5];
    $seconde = $regs[6];
  } else {
    $annee = 1900;
    $mois = 1;
    $jour = 1;
    $heure = 12;
    $minute = 0;
    $seconde = 0;
  }
  return mktime($heure,$minute,$seconde,$mois,$jour,$annee);
}

//-----------------------------------------------------------------------------

// Formate une date au format franais  partir d'un timestamp.
// Ex: "5 Janvier 2003"
function FormateTimestampDate ($timestamp)
{
  setlocale (LC_TIME, "fr_FR");
  return strftime("%d %b %Y", $timestamp);
}

//-----------------------------------------------------------------------------

// Formate une date + heure au format franais  partir d'un timestamp.
// Ex: "5 Janvier 2003  12h03"
function FormateTimestamp ($timestamp)
{
  setlocale (LC_TIME, "fr_FR");
  return strftime("%d %B %Y  %H:%M", $timestamp);
}

//---------------------------------------------------------------------------
?>
