<?php

define('CLASS_XTEMPLATE', TRUE);

/*
	xtemplate class 0.2.4-2
	html generation with templates - fast & easy
	copyright (c) 2000 barnabs debreceni [cranx@scene.hu]
	latest version always available at 
        http://phpclasses.upperdesign.com/browse.html/package/62

	tested with php 3.0.11

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public License
	version 2.1 as published by the Free Software Foundation.

	This library is distributed in the hope that it will be useful, but
        WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details at 
	http://www.gnu.org/copyleft/lgpl.html
  
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

class XTemplate {
	
	var $filecontents = '';							/* raw contents of template file */
	var $blocks = array();							/* unparsed blocks */
	var $parsed_blocks = array();					/* parsed blocks */
	var $block_parse_order = array();				/* block parsing order for recursive parsing (sometimes reverse:) */
	var $sub_blocks = array();						/* store sub-block names for fast resetting */
	var $VARS = array();							/* variables array */
	
	var $file_delim = '/\{FILE\s*\"(.*?)\"\s*\}/m';	/* regexp for file includes */
	var $block_start_delim = '<!-- ';				/* block start delimiter */
	var $block_end_delim = '-->';					/* block end delimiter */
	var $block_start_word = 'BEGIN:';				/* block start word */
	var $block_end_word = 'END:';					/* block end word */
	/* this makes the delimiters look like: <!-- BEGIN: block_name --> if you use my syntax. */
	
	var $NULL_STRING = array('' => '');				/* null string for unassigned vars */
	var $NULL_BLOCK = array('' => '');				/* null string for unassigned blocks */
	var $mainblock = '';
	var $ERROR = '';
	var $AUTORESET = 1;								/* auto-reset sub blocks */
	
	/*
	 * Constructeur
	 */
	function XTemplate ($file, $mainblock = 'main')
	{
		$this->mainblock = $mainblock;
		$this->filecontents = $this->r_getfile($file);	/* read in template file */
		$this->blocks = $this->maketree($this->filecontents, $mainblock);	/* preprocess some stuff */
		$this->scan_globals();
	}
	
	/*
	 *
	 */
	function XTemplateStr($filecontents, $mainblock = 'main')
	{
		$this->mainblock = $mainblock;
		$this->blocks = $this->maketree($filecontents, $mainblock);	/* preprocess some stuff */
		$this->scan_globals();
	}
	
	/*
	 * Assign a variable
	 *
	 * @access public
	 */
	function assign ($name, $val = '')
	{
		if (is_array($name)) {
			while (list($k, $v) = each($name)) {
				$this->VARS[$k] = (string)$v;
			}
			
		} else {
			if (is_array($val)) {
				$this->VARS[$name] = $val;
			} else {
				$this->VARS[$name] = (string)$val;
			}
		}
	}
	
	/*
	 * Parse a block
	 *
	 * @access public
	 */
	function parse ($bname)
	{
		if (!isset($this->blocks[$bname])) {
			$this->blocks[$bname] = '';
			$copy = '';
			$this->set_error('parse: blockname [' . $bname . '] does not exist');
		} else {
			$copy = $this->blocks[$bname];
		}
		
		preg_match_all('/\{([A-Za-z0-9\._]+?)}/', $this->blocks[$bname], $var_array);
		$var_array = $var_array[1];
		while (list($k, $v) = each($var_array)) {
			$sub = explode('.', $v);
			if ($sub[0] == '_BLOCK_') {
				unset($sub[0]);
				$bname2 = implode('.', $sub);
				if (!isset($this->parsed_blocks[$bname2])) {
					$this->parsed_blocks[$bname2] = '';
				}
				$var = $this->parsed_blocks[$bname2];
				$nul = (!isset($this->NULL_BLOCK[$bname2])) ? $this->NULL_BLOCK[''] : $this->NULL_BLOCK[$bname2];
				
			} else {
				$var = $this->VARS;
				while (list($k1, $v1) = each($sub)) {
					$var = (isset($var[$v1])) ? $var[$v1] : '';
				}
				$nul = (!isset($this->NULL_STRING[$v])) ? ($this->NULL_STRING['']) : ($this->NULL_STRING[$v]);
			}
			$var = (!isset($var)) ? $nul : $var;
			$copy = ereg_replace('\{' . $v . '\}', "".$var, $copy);
		}
		if (!isset($this->parsed_blocks[$bname])) {
			$this->parsed_blocks[$bname] = '';
		}
		$this->parsed_blocks[$bname] .= $copy;
		
		// reset sub-blocks 
		if ($this->AUTORESET) {
			if (!empty($this->sub_blocks[$bname])) {
				reset($this->sub_blocks[$bname]);
				while (list($k, $v) = each($this->sub_blocks[$bname])) {
					$this->reset($v);
				}
			}
		}
	}
	
	/*
	 * Return the parsed text for a block, including all sub-blocks.
	 *
	 * @access public
	 */
	function rparse ($bname)
	{
		if (!empty($this->sub_blocks[$bname])) {
			reset($this->sub_blocks[$bname]);
			while (list($k, $v) = each($this->sub_blocks[$bname])) {
				if (!empty($v)) {
					$this->rparse($v, $indent . "\t");
				}
			}
		}
		$this->parse($bname);
	}
	
	/*
	 * Insert a loop (call assign & parse)
	 *
	 * @access public
	 */
	function insert_loop ($bname, $var, $value = '')
	{
		$this->assign($var, $value);		
		$this->parse($bname);
	}
	
	/*
	 * Return the parsed text for a block
	 *
	 * @access public
	 */
	function text ($bname)
	{
		if (!isset($bname)) {
			$bname = $this->mainblock;
		}
		
		return $this->parsed_blocks[$bname];
	}
	
	/*
	 * Print the parsed text
	 *
	 * @access public
	 */
	function out ($bname)
	{
		echo $this->text($bname);
	}
	
	/*
	 * Reset the parsed text
	 *
	 * @access public
	 */
	function reset ($bname)
	{
		$this->parsed_blocks[$bname] = '';
	}
	
	/*
	 * Return true if block was parsed, false if not
	 *
	 * @access public
	 */
	function parsed ($bname)
	{
		return (!empty($this->parsed_blocks[$bname]));
	}
	
	/*
	 * Set the string to replace in case the var was not assigned
	 *
	 * @access public
	 */
	function SetNullString ($str, $varname = '')
	{
		$this->NULL_STRING[$varname] = $str;
	}
	
	/*
	 * Set the string to replace in case the block was not parsed
	 *
	 * @access public
	 */
	function SetNullBlock ($str, $bname = '')
	{
		$this->NULL_BLOCK[$bname] = $str;
	}
	
	/*
	 * Set AUTORESET to 1. (default is 1)
	 * if set to 1, parse() automatically resets the parsed blocks' sub blocks
	 * (for multiple level blocks)
	 *
	 * @access public
	 */
	function set_autoreset ()
	{
		$this->AUTORESET = 1;
	}
	
	/*
	 * Set AUTORESET to 0. (default is 1)
	 * if set to 1, parse() automatically resets the parsed blocks' sub blocks
	 * (for multiple level blocks)
	 *
	 * @access public
	 */
	function clear_autoreset ()
	{
		$this->AUTORESET = 0;
	}
	
	/*
	 * Scan global variables
	 *
	 * @access public
	 */
	function scan_globals ()
	{
		global $BLOC;
		
		reset($GLOBALS);
		while (list($k, $v) = each($GLOBALS)) {
			$GLOB[$k] = $v;
		}
		
		$this->assign('PHP', $GLOB);	/* access global variables as {PHP.HTTP_HOST} in your template! */
		$this->assign('BLOC', $BLOC);	/* access global variables as {PHP.HTTP_HOST} in your template! */
	}
	
	/*
	 * Generate the array containing to-be-parsed stuff:
	 * $blocks["main"], $blocks["main.table"], $blocks["main.table.row"], etc.
	 * also builds the reverse parse order.
	 *
	 * @access private
	 *
	 * @param string
	 * @param string
	 *
	 * @return array string 'To-be-parsed stuff
	 */
	function maketree ($content, $block)
	{
		$content = explode($this->block_start_delim, $content);
		$level = 0;
		$block_names = array();
		$blocks = array();
		reset($content);
		
		while(list($k, $v) = each($content)) {
			$pattern = '(' . $this->block_start_word . '|' . $this->block_end_word . ')[[:blank:]]*([0-9a-zA-Z\_]+)[[:blank:]]*' . $this->block_end_delim . '(.*)';
			
			if (eregi($pattern, $v, $res)) {
				if ($res[1] == $this->block_start_word) {
					$parent_name = implode('.', $block_names);
					$block_names[++$level] = $res[2];								/* add one level - array("main", "table", "row")*/
					$cur_block_name = implode('.', $block_names);					/* make block name (main.table.row) */
					$this->block_parse_order[] = $cur_block_name;					/* build block parsing order (reverse) */
					if (!isset($blocks[$cur_block_name])) {
						$blocks[$cur_block_name] = '';
					}
					$blocks[$cur_block_name] .= $res[3];							/* add contents */
					$blocks[$parent_name] .= '{_BLOCK_.' . $cur_block_name . '}';	/* add {_BLOCK_.blockname} string to parent block */
					$this->sub_blocks[$parent_name][] = $cur_block_name;			/* store sub block names for autoresetting and recursive parsing */
					$this->sub_blocks[$cur_block_name][] = '';						/* store sub block names for autoresetting */
					
				} elseif ($res[1] == $this->block_end_word) {
					unset($block_names[$level--]);
					$parent_name = implode('.', $block_names);
					$blocks[$parent_name] .= $res[3];								/* add rest of block to parent block */
				}
				
			} else {																/* no block delimiters found */
				$cur_block_name = implode('.', $block_names);
				if (!isset($blocks[$cur_block_name])) {
					$blocks[$cur_block_name] = '';
				}
				$blocks[$cur_block_name] .= $this->block_start_delim . $v;
			}
		}
		
		return $blocks;	
	}
	
	/*
	 * Get error
	 *
	 * @access private
	 *
	 * @return string 'Erreur
	 */
	function get_error ()
	{
		return ($this->ERROR == '') ? 0 : $this->ERROR;
	}
	
	/*
	 * Set error
	 *
	 * @access private
	 *
	 * @param string 'Erreur
	 */
	function set_error ($str)
	{
	    if (1) echo "<p>$str</p>\n";
	    $this->ERROR = $str;
	}
	
	/*
	 * Return the contents of a file
	 *
	 * @access private
	 *
	 * @param string 'Path to file
	 *
	 * @return string 'Content of the file
	 */
	function getfile ($file)
	{
		global $rootpath;
		
		$file = str_replace('{PHP.rootpath}', $rootpath, $file);
		if (!isset($file)) {
			$this->set_error('!isset file name!');
			return '';
		}
		
		if (is_file($file)) {
			if (!($fh = fopen($file, 'r'))) {
				$this->set_error('Cannot open file: ' . $file);
				return '';
			}
			$file_text = fread($fh, filesize($file));
			fclose($fh);
		} else {
			$this->set_error('[' . $file . '] does not exist');
			$file_text = '<b>__XTemplate fatal error: file [' . $file . '] does not exist__</b><br/>';
		}
		
		return $file_text;
	}
	
	/*
	 * Recursively get the content of a file with {FILE "filename.tpl"} directives
	 *
	 * @access private
	 *
	 * @param string 'Path to file
	 *
	 * @return string 'Content of the file
	 */
	function r_getfile ($file)
	{
		$text = $this->getfile($file);
		while (preg_match($this->file_delim, $text, $res)) {
			$subtext = $this->getfile($res[1]);
			$text = ereg_replace($res[0], $subtext, $text);
		}
		
		return $text;
	}
}

?>