<?php
class MediaDB
{
    var $dir;
    var $sql_table;
    var $image_thumbnail_cfg;

    function MediaDB($sql_table, $dir) {
        $this->sql_table = $sql_table;
        $this->dir=$dir;
        $this->image_thumbnail_cfg = Array(
            'width'=>150,
            'height'=>100,
            'jpeg_quality'=>60);
    }

    function upload_file($fichier) {
        if (!file_exists($fichier['tmp_name'])) {
            Erreur('Erreur interne : impossible de retrouver l\'image ! Le fichier est peut-tre trop gros ?');
            return false;
        }
        $destination = $this->dir .'/'. $fichier['name'];

        // Check if destination already exits
        if (file_exists($destination)) {
            Erreur('Le mdia "<em>'.$destination . '</em>" existe dj !');
            return false;
        }

        if (!move_uploaded_file($fichier['tmp_name'], $destination)) {
            Erreur('Impossible de deplacer ' . $fichier['tmp_name']
                    .' dans '.$destination.' !');
            return FALSE;
        }

        return Array(
            'filename'=>$fichier['name'],
            'dest'=>$destination);
    }

    function is_image($type) {
        return ereg('^(gif|png|jpeg|image)$', $type);
    }

    function is_flash($type) {
        return ereg('^swf$', $type);
    }

    function get_media_type($filename) {
        return q_read_field(
            'type',
            $this->sql_table,
            Array('filename'=>$filename));
    }

    function remove_media() {
        $filename = HtmlLitVar('filename');
        $thumb = NomfichSansExt($filename).'.jpg';
        if ($filename == '') return;

        $type = $this->get_media_type($filename);

        $full_filename = $this->dir.'/'.$filename;
        Message('Supprime le fichier <em>'.$full_filename.'</em>.');
        unlink($full_filename);

        if ($this->is_image($type)) {
            $full_filename = $this->dir.'/thumbnail/'.$thumb;
            Message('Supprime le fichier <em>'.$full_filename.'</em>.');
            unlink($full_filename);            
        }
        
        q_delete(
            $this->sql_table,
            Array('filename'=>$filename));
    }

    function add_media() {
        $file = $_FILES['new_media'];
        $type = mime_identify_file($file['tmp_name'], $file['name']);
        if ($type === false) $type = 'file';
        $is_image = $this->is_image($type);
        $thumb_dir = $this->dir.'/thumbnail';
        if ($is_image && !is_dir($thumb_dir) && !mkdir($thumb_dir, 0700)) {
            Erreur('Impossible de crer le rpertoire "'.$thumb_dir.'" !');
            return;
        }
        
        $dst = $this->upload_file($file);
        if (!$dst) return;
        if ($is_image) {
            $cfg = $this->image_thumbnail_cfg;
            image_generate_thumbnail(
                $this->dir, $thumb_dir, $dst['filename'],
                $cfg['width'], $cfg['height'], $cfg['jpeg_quality']);
        }
        q_insert(
            $this->sql_table,
            Array('type'=>$type, 'filename'=>$dst['filename']));
    }

    function out_db() {
        $xtpl=new XTemplate('./templates/F/mediadb/db.xtpl');
        $r = q_select(
            Array('no','type','filename'),
            $this->sql_table,
            false,
            false,
            'filename');
        $nb_medias = q_num_rows($r);
        while ($row = q_fetch_assoc($r)) {
            $type = $row['type'];
            $filename = $row['filename'];
            if ($this->is_image($type)) {
                $icon = NomfichSansExt($filename).'.jpg';
                $xtpl->assign('icon', $this->dir.'/thumbnail/'.$icon);
            } else {
                $xtpl->assign('icon', './images/mime/'.$type.'.png');
            }
            $xtpl->assign('filename',$filename);
            $xtpl->parse('main.list.media');
        }
        if (0 < $nb_medias) {
            $xtpl->parse('main.list');
        } else {
            $xtpl->parse('main.empty_list');            
        }
        $xtpl->parse('main.add_media_form');            
        $xtpl->parse('main');
        return $xtpl->text('main');
    }

    function out_wysiwyg() {
        $xtpl=new XTemplate('./templates/F/mediadb/wysiwyg.xtpl');
        $r = q_select(
            Array('no','type','filename'),
            $this->sql_table,
            false,
            false,
            'filename');
        $nb_medias = q_num_rows($r);
        $virgule = false;
        while ($row = q_fetch_assoc($r)) {
            $type = $row['type'];
            $filename = $row['filename'];
            $id = strtr($filename, '\'/-.','____');
            $is_img = $this->is_image($type);
            $is_flash = $this->is_flash($type);

            if ($is_img) {
                $icon = NomfichSansExt($filename).'.jpg';
                $icon = $this->dir.'/thumbnail/'.$icon;
            } else {
                $icon = './images/mime/'.$type.'.png';
            }
            $xtpl->assign('icon', $icon);
            $xtpl->assign('filename',$filename);
            $xtpl->assign('id',$id);
            $xtpl->parse('main.list.media');

            if ($is_img) {
                $html  = '<img src="'.$this->dir.'/'.$filename;
                $html .= '" style="vertical-align: middle;" />';
            } else if ($is_flash) {
                $url = $this->dir.'/'.$filename;
                $quality = "high";
                $bgcolor = 'white';
                $width = '300px';
                $height = '200px';
                
                $html  = '<object classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" ';
                $html .= 'codebase="http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/';
                $html .= 'swflash.cab#version=7,0,0,0" height="'.$height.'" width="'.$width.'">';
                $html .= '  <param name="movie" value="'.$url.'">';
                $html .= '	<param name="quality" value="'.$quality.'">';
                $html .= '	<param name="bgcolor" value="'.$bgcolor.'">';
                $html .= '	<embed src="'.$url.'" quality="'.$quality.'" bgcolor="'.$bgcolor;
                $html .= '" type="application/x-shockwave-flash" pluginspage=';
                $html .= '"http://www.macromedia.com/go/getflashplayer" height="'.$height.'" width="'.$width.'">';
                $html .= '</object>';
            } else {
                $url = $this->dir.'/'.$filename;
                $html  = '<a href="'.$url.'">';
                $html .= '<p><img src="'.$icon;
                $html .= '" style="vertical-align: middle;" />';
                $html .= $filename.'</p>';
                $html .= '</a>';
            }
            //$xtpl->assign('value', htmlspecialchars($html)); 
            $html = str_replace('"','\\\"', $html);
            $html = str_replace("'",'\\\'', $html); // ???
            $xtpl->assign('value', $html); 
            $xtpl->parse('main.javascript.list_item');
        }
        if (0 < $nb_medias) {
            $xtpl->parse('main.javascript');
            $xtpl->parse('main.list');
        } else {
            $xtpl->parse('main.empty_list');            
        }
        $xtpl->parse('main.add_media_form');            
        $xtpl->parse('main');
        return $xtpl->text('main');
    }
}

?>
