<?php
//-----------------------------------------------------------------------------

function message_setup($txt,$info_sup=false) {
  echo '<p class="message">'.$txt."</p>\n";
  if ($info_sup) echo $info_sup."\n";
}

//-----------------------------------------------------------------------------

function erreur_setup($txt,$info_sup=false) {
  echo '<p class="erreur">'.$txt."</p>\n";
  if ($info_sup) echo $info_sup."\n";
}

//-----------------------------------------------------------------------------

// Excute une commande en y ajoutant " 2>&1" pour attraper les erreurs
// Renvoie un tableau associatif :
//   'exit_code' : code d'erreur
//   'content'   : retour du programme
function execute($cmd, $msg_erreur=false, $info_erreur=false) {
    ob_start();
    passthru($cmd.' 2>&1', &$exit_code);
    $content = ob_get_contents();
    ob_end_clean();

    // Vrifie le code d'erreur
    if (($exit_code != 0) && ($msg_erreur != '')) {
        if ($info_erreur) $txt  = "<p>".$info_erreur."</p>\n"; else $txt = '';
        $txt .= "<p>Commande = ".htmlspecialchars($cmd)."</p>\n";
        $txt .= "<p>Sortie =</p>\n<pre>".htmlspecialchars($content)."</pre>"; 
        erreur_setup(
                'Erreur (code '.$exit_code.')  lors de '.$msg_erreur,
                $txt);
    }
    return Array(
        'exit_code' => $exit_code, 
        'content' => $content);
}

//-----------------------------------------------------------------------------

// Lit une variable HTML dans $POST ou $GET
function HtmlLitVar($nom)
{
  if (array_key_exists($nom,$_POST))
    return stripcslashes($_POST[$nom]);
  if (array_key_exists($nom,$_GET))
    return stripcslashes($_GET[$nom]);
  return false;
}

//-----------------------------------------------------------------------------

function EntreeValide(&$valeur, $description) {
  $valeur = trim($valeur);
  $ok = ($valeur != '');
  if (!$ok) {
    erreur_setup('Veuillez renseignez le champ <em>'.$description.'</em> !');
  }
  return $ok;
}

//-----------------------------------------------------------------------------

function SauveEntree($sortie, $champs, $_POST) {
    $f = fopen ($sortie,'w');
    if (!$f) return false;
    fputs ($f, '<'."?php\n");
    foreach ($champs as $name => $val) {
      fputs ($f, '$'.$name.'=\''.$_POST[$name]."';\n");
    }
    fputs ($f, '?'.">\n");
    return true;
}

//-----------------------------------------------------------------------------

/**
 * Removes comment lines and splits up large sql files into individual queries
 *
 * Last revision: September 23, 2001 - gandon
 *
 * @param   array    the splitted sql commands
 * @param   string   the sql commands
 * @param   integer  the MySQL release number (because certains php3 versions
 *                   can't get the value of a constant from within a function)
 *
 * @return  boolean  always true
 *
 * @access  public
 */
function PMA_splitSqlFile(&$ret, $sql, $release)
{
    $sql          = trim($sql);
    $sql_len      = strlen($sql);
    $char         = '';
    $string_start = '';
    $in_string    = FALSE;
    $time0        = time();

    for ($i = 0; $i < $sql_len; ++$i) {
        $char = $sql[$i];

        // We are in a string, check for not escaped end of strings except for
        // backquotes that can't be escaped
        if ($in_string) {
            for (;;) {
                $i         = strpos($sql, $string_start, $i);
                // No end of string found -> add the current substring to the
                // returned array
                if (!$i) {
                    $ret[] = $sql;
                    return TRUE;
                }
                // Backquotes or no backslashes before quotes: it's indeed the
                // end of the string -> exit the loop
                else if ($string_start == '`' || $sql[$i-1] != '\\') {
                    $string_start      = '';
                    $in_string         = FALSE;
                    break;
                }
                // one or more Backslashes before the presumed end of string...
                else {
                    // ... first checks for escaped backslashes
                    $j                     = 2;
                    $escaped_backslash     = FALSE;
                    while ($i-$j > 0 && $sql[$i-$j] == '\\') {
                        $escaped_backslash = !$escaped_backslash;
                        $j++;
                    }
                    // ... if escaped backslashes: it's really the end of the
                    // string -> exit the loop
                    if ($escaped_backslash) {
                        $string_start  = '';
                        $in_string     = FALSE;
                        break;
                    }
                    // ... else loop
                    else {
                        $i++;
                    }
                } // end if...elseif...else
            } // end for
        } // end if (in string)

        // We are not in a string, first check for delimiter...
        else if ($char == ';') {
            // if delimiter found, add the parsed part to the returned array
            $ret[]      = substr($sql, 0, $i);
            $sql        = ltrim(substr($sql, min($i + 1, $sql_len)));
            $sql_len    = strlen($sql);
            if ($sql_len) {
                $i      = -1;
            } else {
                // The submited statement(s) end(s) here
                return TRUE;
            }
        } // end else if (is delimiter)

        // ... then check for start of a string,...
        else if (($char == '"') || ($char == '\'') || ($char == '`')) {
            $in_string    = TRUE;
            $string_start = $char;
        } // end else if (is start of string)

        // ... for start of a comment (and remove this comment if found)...
        else if ($char == '#'
                 || ($char == ' ' && $i > 1 && $sql[$i-2] . $sql[$i-1] == '--')) {
            // starting position of the comment depends on the comment type
            $start_of_comment = (($sql[$i] == '#') ? $i : $i-2);
            // if no "\n" exits in the remaining string, checks for "\r"
            // (Mac eol style)
            $end_of_comment   = (strpos(' ' . $sql, "\012", $i+2))
                              ? strpos(' ' . $sql, "\012", $i+2)
                              : strpos(' ' . $sql, "\015", $i+2);
            if (!$end_of_comment) {
                // no eol found after '#', add the parsed part to the returned
                // array if required and exit
                if ($start_of_comment > 0) {
                    $ret[]    = trim(substr($sql, 0, $start_of_comment));
                }
                return TRUE;
            } else {
                $sql          = substr($sql, 0, $start_of_comment)
                              . ltrim(substr($sql, $end_of_comment));
                $sql_len      = strlen($sql);
                $i--;
            } // end if...else
        } // end else if (is comment)

        // ... and finally disactivate the "/*!...*/" syntax if MySQL < 3.22.07
        else if ($release < 32270
                 && ($char == '!' && $i > 1  && $sql[$i-2] . $sql[$i-1] == '/*')) {
            $sql[$i] = ' ';
        } // end else if

        // loic1: send a fake header each 30 sec. to bypass browser timeout
        $time1     = time();
        if ($time1 >= $time0 + 30) {
            $time0 = $time1;
            header('X-pmaPing: Pong');
        } // end if
    } // end for

    // add any rest to the returned array
    if (!empty($sql) && preg_match('@[^[:space:]]+@', $sql)) {
        $ret[] = $sql;
    }

    return TRUE;
} // end of the 'PMA_splitSqlFile()' function

//-----------------------------------------------------------------------------

// Renvoie la version MySQL (il faut tre connect !) : 
// entier de la forme MmmPP avec M=Major, mm=minor, PP=patch level
// Expl: 4.1 -> 40100
function LitVersionMySQL() {
  $result = mysql_query('SELECT VERSION() AS version');
  $row   = mysql_fetch_array($result);
  $match = explode('.', $row['version']);
  mysql_free_result($result);

  if (!isset($match) || !isset($match[0])) $match[0] = 3;
  if (!isset($match[1])) $match[1] = 23;
  if (!isset($match[2])) $match[2] = 32;

  if(!isset($row)) $row['version'] = '3.23.32';

  return (int)sprintf('%d%02d%02d', $match[0], $match[1], intval($match[2]));
  // return $row['version'];
}

//-----------------------------------------------------------------------------

function Castor_ListeModules($dir)
{
    $handle = @opendir($dir);
    if (!$handle) 
      ErreurFatale ('Impossible d\'ouvrir le rpertoire "'.
		    $dir.'" !');

    $modules = array();
    while (($item = readdir($handle)) !== false) {

	// Exclu '.', '..', les rpertoires
	// et webmodule.php
	if (($item=='.') || ($item=='..')) continue;
	if (!is_file($dir.$item)) continue;

	$item = explode('.',$item);

	if ($item[0]=='webmodule') continue;
	if ($item[1]!='php') continue;

	$modules[] = $item[0];
    }
    closedir($handle);
    return $modules;
}

//-----------------------------------------------------------------------------

// Check mode of file for PHP user
// Mode should be in form "rwx" ("---", "rw-", ...)
function check_file_mode($file, $mode) {
    if (($mode[0]=='r') && !is_readable($file)) return false;
    if (($mode[1]=='w') && !is_writable($file)) return false;
    if (($mode[2]=='x') && !is_executable($file)) return false;
    return true;
}

//-----------------------------------------------------------------------------

// Check directory of file (recursive) for PHP user
// Mode should be in form "rwx" ("---", "rw-", ...)
function check_dir_mode ($dir, $mode) {
    if (!check_file_mode($dir,$mode)) return false;
    $ok = true;
    $d = @opendir($dir);
    if ($d === false) return false;
    while ($ok && (($f = readdir($d)) !== false)) {
        if (($f == '.') || ($f == '..')) continue;
        $file = $dir.'/'.$f;
        if (is_dir($file)) {
            if (!check_dir_mode($file, $mode)) $ok = false;
        } else if (!check_file_mode($file, $mode)) {
            $ok = false;
        }
    }
    closedir($d);
    return $ok;
}

//-----------------------------------------------------------------------------
?>
